-- @(#) dopop.sql 1.1 0 08/09/10 18:32:23
-- @(#) 
--
-- Copyright (c) 2008 IBM. All rights reserved

-- very useful to give us a reasonable execution plan for the following queries. This avoids
-- nested loop hell, and a performance increase of >10x

-- That is an issue in 10g of auto stat collection,
-- Use Prospect own stat collection instead
-- disable: ANALYZE TABLE ppconfig_stage COMPUTE STATISTICS;

/*
DECLARE
  username VARCHAR2(256);
BEGIN
  SELECT user INTO username FROM DUAL;

    dbms_stats.gather_table_stats(
      ownname     => username,
      tabname     => 'PPCONFIG_STAGE',
      cascade     => TRUE
    );
END;
/
*/

-- End of RP14 (20080712): Use Prospect own stat collection instead


TRUNCATE TABLE jloader_pdo_table_tmp;
TRUNCATE TABLE jloader_pdo_colmap_tmp;
TRUNCATE TABLE jloader_pdo_record_tmp;

-- Firstly clear out any data currently associated with this version. This is important
-- because we need to deal with rows potentially being removed from the configuration
EXEC jloader_pdo.delete_version('&1');

create or replace view lu_utran_rnc_cn_counters as
select fd.field_id, fd.entity_id,
       fd_sf.datavalue sourcefield,
       decode (fd_ss.datavalue,
               'FddCell', 'UtranCell',
               'RncEquipment', 'RncFunction',
               fd_ss.datavalue
              ) sourcesection
  from wmd_field_data fd, wmd_field_data_set fds,
       wmd_field_data fd_sf, wmd_field_data fd_ss
 where fd.dataset_id = fds.dataset_id and
       fds.dataset_title = 'DataSource' and
       fd.datavalue = 'RNC C-Node' and
       fd_sf.field_id = fd.field_id and
       fd_sf.dataset_id = 
         (select dataset_id from wmd_field_data_set
           where dataset_title = 'SourceField')
       and
       fd_ss.field_id = fd.field_id and
       fd_ss.dataset_id =
         (select dataset_id from wmd_field_data_set
           where dataset_title = 'SourceSection');

-- ------------------------------------------------------------------
-- Create a list of all counter groups for the RNC CN file type.
-- Not all these counter groups (record_types, in old jloader parlance)
-- will be mapped to physical tables, but we are modeling the input to
-- JLoader here, so we need to have a 
-- complete description of everything that JLoader can see.
-- ------------------------------------------------------------------
/*
INSERT INTO jloader_pdo_record_tmp (version_name, record_type, dn_rewrite_func)
     SELECT '&1' version_name,
            pp.measurement_group,
            min(pp.rewrite_function) dn_rewrite_func
       FROM ppconfig_stage pp
   GROUP BY pp.measurement_group;
*/

prompt Populating jloader_pdo_record_tmp
insert into jloader_pdo_record_tmp
       (version_name, record_type, dn_rewrite_func)
select '&1' version_name,
       rnc_cn_entities.sourcesection,
       'rewrite_' || rnc_cn_entities.sourcesection
from
   (select distinct sourcesection 
      from lu_utran_rnc_cn_counters
   ) rnc_cn_entities;
/*
fd.datavalue measurement_group
      from wmd_field_data fd, wmd_field_data_set fds,
           (select fd.field_id rnc_cn_field_id
              from wmd_field_data fd, wmd_field_data_set fds
             where fd.dataset_id = fds.dataset_id and
                   fds.dataset_title = 'DataSource' and
                   fd.datavalue = 'RNC C-Node'
           ) rnc_cn_fields
     where fd.field_id = rnc_cn_fields.rnc_cn_field_id and
           fd.dataset_id = fds.dataset_id and
           fds.dataset_title = 'SourceSection'
   ) rnc_cn_entities;
*/


-- ---------------------------------------------------------------------
-- The following insert creates an entry for every table that we are using
-- in our mapping. This information is used to create the necessary
-- wml_load entries that support Jloader.
-- ---------------------------------------------------------------------

prompt Populating jloader_pdo_table_tmp
INSERT INTO jloader_pdo_table_tmp
       (version_name, table_name, entity_name, record_type, post_load_proc, max_batch_count,
        date_format, dn_print_class, dn_rewrite_func, id_column_name, needs_entity_map, needs_rg_reap, needs_traf,
        needs_truncate, needs_instance, needs_trace, needs_change_event, needs_last_seen,
        find_instance_func, instance_lookup_func, instance_name_colname)
SELECT '&1' version_name,
       tables.table_name,
       tables.entity_name,
       tables.counter_group,
       'wml_load.do_load' post_load_proc,
       200 max_batch_count,
       'yyyy-MM-dd HH:mm:ss' date_format,
       'com.watchmark.jloader.processor.vendor.lucent.umts.RncCnDNPrinter' dn_print_class,
       tables.rewrite_function dn_rewrite_func,
       uic.column_name id_column_name,
       'N' needs_entity_map,
       'N' needs_rg_reap,
       'Y' needs_traf,
       'N' needs_truncate,
       'Y' needs_instance,
       'N' needs_trace,
       'N' needs_change_event,
       'N' needs_last_seen,
       'wm_find_instance.find_instance' find_instance_func,
       ''  instance_lookup_func,
       '' instance_name_colname
  FROM
         (SELECT 
               e.entity_name,
               dt.table_name,
               pp.record_type counter_group,
               pp.dn_rewrite_func rewrite_function
          FROM wmn_entity e,
               wmd_dict_table dt,
               (select r.record_type, r.dn_rewrite_func,
                       fd.entity_id
                  from jloader_pdo_record_tmp r,
                       wmd_field_data fd
                 where fd.datavalue = r.record_type
                   and fd.dataset_id =
                       (select dataset_id from wmd_field_data_set
                         where dataset_title = 'SourceSection')
                group by r.record_type, r.dn_rewrite_func, fd.entity_id
               ) pp
         WHERE e.entity_id = pp.entity_id
           and e.entity_id = dt.entity_id
           AND dt.is_time_series = 'Y'
           AND dt.summary_id = 0 
      GROUP BY e.entity_name, dt.table_name, pp.record_type, pp.dn_rewrite_func) tables,
        user_ind_columns uic
 WHERE uic.table_name = tables.table_name 
   AND uic.index_name LIKE 'PK%' 
   AND uic.column_position <= 2
   AND uic.column_name != 'TSTAMP';



-- -----------------------------------------------------------------------------------------------------
-- So basically what we want here is for every table in
-- the tmp table above, we need to map all of its columns onto fields in the counter group (or at
-- least as many as we can). At the same time, we need to add a couple of extra fields to
-- represent the id and tstamp columns. This means we need to union some selects
-- Note the outer join - this means we load all the counters defined in ppConfig, even those that
-- are not mapped (these counters might be used in rollups...but we still need their meta-data)
-- -----------------------------------------------------------------------------------------------------

prompt Populating jloader_pdo_colmap_tmp
INSERT INTO jloader_pdo_colmap_tmp
           (version_name, group_name, table_name, column_name, counter_name, 
            array_index, accumulator, reset_test, bit_size)
SELECT '&1' version_name, pp.sourcesection, tc.table_name, tc.table_column, 
       DECODE(UPPER(pp.sourcefield), 'PERLENSEC',     ':granularityPeriod', 
                                      'PERLENSEC_K',   ':granularityPeriod',
                                      'RNC_RELEASE',   ':release',
                                      'NODEB_RELEASE', ':release',
                                                       pp.sourcefield) counter_name,
       NULL array_index,
       'N' accumulator,
       'N' reset_test,
       0 bit_size
FROM lu_utran_rnc_cn_counters pp,
(
SELECT e.entity_id, df.field_id, dt.table_name, df.table_column
  FROM wmn_entity e,
       wmd_dict_field df,
       wmd_dict_table dt
 WHERE dt.table_id = df.table_id
   AND dt.is_time_series = 'Y'
   AND dt.summary_id = 0
   AND df.entity_id = e.entity_id
) tc
WHERE pp.entity_id = tc.entity_id(+)
  AND pp.field_id = tc.field_id(+)
UNION ALL
SELECT tt.version_name, tt.record_type, tt.table_name, tt.id_column_name, ':fdn', NULL, 'N', 'N', NULL
  FROM jloader_pdo_table_tmp tt
UNION ALL
SELECT tt.version_name, tt.record_type, tt.table_name, 'TSTAMP', ':tstamp', NULL, 'N', 'N', NULL
  FROM jloader_pdo_table_tmp tt
;

-- OK, now we can use the mapping info we loaded into the temp tables
-- to create the actual meta-data
prompt Copying _tmp tables to Jloader metadata tables...
exec jloader_pdo.load_from_record_tmp
exec jloader_pdo.load_from_table_tmp
exec jloader_pdo_colmap_api.add_mappings_from_tmp
COMMIT;
prompt dopop.sql finished.


