# @(#) PLM_LUCENT_UMTS_RNC.pm 1.5.0 05/10/14 10:18:04
# @(#) 

## Lucent UMTS RNC XML data hash module.

use strict;

sub build_data_hash {

  my ($ppConfig, $output, $insep, $config) = @_;

  my ($file_content, $meas_time, $cnt_name, $cnt_value, $perlen, $moid, $moid_entity, $moid_value );
  my ($RNC_name, $RNC_id);

  my $create_jobidx_cache= {}; 
  # We have to create job idx to fit with the Prospect hierarchy,
  # $create_jobidx_cache caches the results (!) so we don't have to
  # call the create subroutine more than once for a given MOID.

  my $create_jobidx_or_not= {};
  # $create_jobidx_or_not keeps track of whether we succeeded in creating
  # a given MOID or not. 
  #
  # The value will be one of -
  # undef - this job idx has not been created.
  # 1     - this job idx has been created successfuly
  # 0     - this job idx has failed to be created

  my $moid_separator= $ppConfig->{CONFIG}{MOIDSEPARATOR};

  my %missing_counters;

  my $skip_ahead = "yes";
  my $output_ref;

  while ($file_content = <STDIN>) {
    chomp($file_content);

    my ($line_type, $rest)= $file_content =~ /^([^$insep]*)$insep(.*)$/o;
    if ($line_type eq "FILENAME") {
      $skip_ahead= "no";

      my $file_name = $rest;
      
      ($RNC_name, $RNC_id, $meas_time) = parse_filename($file_name, $config, $ppConfig);

      $ppConfig->{RNC_NAME} = $RNC_name;

      if (not defined $RNC_id or not defined $meas_time)
	{
	  # No point parsing this file if we cannot extract the RNC ID or the MEAS_TIME
	  messageHandler($config, 1, "Skipping ...");
	  $skip_ahead="yes";
	} # endif not defined meas_time

    } elsif ($line_type eq "SN") {
      # Can get timestamp from SN but it is extracted from the filename.
      # Even though we do nothing with the SN line,
      # we have to test for it. Otherwise "SN" looks like a counter,
      # and the code that handles counters doesn't know what to do with it.
    } elsif ($line_type eq "LABEL") {
      my $RNC_name = $rest;

      $RNC_name =~ tr/-!/~|/;

      $ppConfig->{RNC_LABEL} = $RNC_name;
    } else {
      # $line_type is not "SN" or "FILENAME"
      next if $skip_ahead eq "yes";
      if ($line_type eq "MOID") {
	($moid, $perlen)= split /$insep/o, $rest;
	($moid_entity, $moid_value) = $moid =~ /(\w+)=(.*)/;
	$output_ref= \%{$output->{$meas_time}{$perlen}};
      } else {
	($cnt_name, $cnt_value)= ($line_type, $rest);
	# If the first field on the line is not "FILENAME, "LABEL", "SN" or "MOID", 
	# it must be a counter name.

        my ($jobidx, $meas_group)= (undef, undef);

	if (exists $ppConfig->{COUNTER_TO_GROUP}{$moid_entity}{$cnt_name}) 
	    {
	      $meas_group = $ppConfig->{COUNTER_TO_GROUP}{$moid_entity}{$cnt_name};

	      if (not exists $create_jobidx_or_not->{$meas_group}{$moid_value}) {

		# Assume that we can't create the job idx until we've succeeded
		# in rewriting it.
		$create_jobidx_or_not->{$meas_group}{$moid_value} = 0;
		if (not defined $moid_value or $moid_value eq '') {
		  messageHandler
		    ($config, 1,
		     "Could not create full key because moid value is undefined or null");
		} else {

		  my $module = $ppConfig->{MODLIST}{$meas_group};

		  # Could use 'eval' statement but as there are
		  # only four modules an 'if' executes quicker.
		  if ($module eq "create_idx_UtranCell") {
		    if (length($moid_value) < 2) {
		      messageHandler
			($config, 1, 
			 "UtranCell moid too short to extract NodeB id.  Moid : $moid_value");
		    } else {
		      my $NodeB_id = substr($moid_value, 0, -1);
		      $jobidx= "$RNC_id-$NodeB_id-$moid_value";

		    }		    
		  } elsif ($module eq "create_idx_NodeB" || 
			   $module eq "create_idx_NeighborRNC") {
		    $jobidx = $RNC_id . "-" . $moid_value;
		    
		  } elsif ($module eq "none") {
		    $jobidx = $RNC_id;
		    
		  } else {
		    messageHandler($config , 1, "Unknown module : ", $module);
		  }

		  if (defined $jobidx) {
		    $create_jobidx_or_not->{$meas_group}{$moid_value} = 1;
		    # Place it in the cache so we can utilise it next time
		    $create_jobidx_cache->{$meas_group}{$moid_value} = $jobidx;
		  } else {
		    messageHandler($config, 1,
				   "Could not create full key for counter : ", $cnt_name
				 , "Measurement group ", $meas_group);
		  } # if defined $jobidx
		} # if not defined $moid_value ...
	      } elsif ($create_jobidx_or_not->{$meas_group}{$moid_value} == 1) {
		$jobidx = $create_jobidx_cache->{$meas_group}{$moid_value};
	      } # if rewite jobidx
	    } # if exists $ppConfig-> ...
	if (defined $jobidx) {
	  my $field_pos= $ppConfig->{COUNTGROUPFIELD}{$meas_group}{$cnt_name};
	  $output_ref->{$meas_group}{$jobidx}{$field_pos} = $cnt_value;
	} else {
	  # Found counter in data which we do not know about
	  unless (exists $missing_counters{$moid_entity}{$cnt_name}) {
	    messageHandler($config, 1, "No mapping found for counter : ",
			   $cnt_name,", moid : ",$moid_entity);
	    $missing_counters{$moid_entity}{$cnt_name}= 1;
	    # Complain about each unknown counter only the first time we see it.
	  }
	} # if COUNTER TO GROUP mapping 
      } # if $line_type eq "MOID"
    } # if $line_type eq "SN"
  } # while

  # SEAde57340 - calculate complicated UDCs that do not work in Prospect Web
  for my $meas_time (keys %{$output}) {
    for my $perlen (keys %{$output->{$meas_time}}) {
      if (exists $output->{$meas_time}{$perlen}{UtranCell2}) {

        my @required_counters = ("NumActRABMean_CSV12",
                                 "NumActRABMean_CSD",
                                 "RAB_RelCS_Voice_CauseRLF",
                                 "RAB_Rel_Drop_sum",
                                 "RAB_RelCS_Data_CauseRLF",
                                 "RAB_RelPS_DCH_CauseRLF",
                                 "RAB_RelPS_HSDSCH_CauseRLF",
                                 "RAB_RelPS_HSDSCH_CauseRLF_ReconfFail",
				);

        for my $jobidx (keys %{$output->{$meas_time}{$perlen}{UtranCell2}}) {

	  my $output_ref= \%{$output->{$meas_time}{$perlen}{UtranCell2}{$jobidx}};
          my %values;

          for my $cnt_name (@required_counters) {
            my $field_pos=$ppConfig->{COUNTDDGROUPFIELD}{UtranCell2}{$cnt_name};

            if (exists $output_ref->{$field_pos} and
                       $output_ref->{$field_pos} ne '') { 
              $values{$cnt_name} = $output_ref->{$field_pos};
            } else {
              $values{$cnt_name} = 0;
            }
          }

          my $field_pos;

          # NumActRABs_CSV12_Accumulated
          $field_pos=$ppConfig->{COUNTDDGROUPFIELD}{UtranCell2}{NumActRABs_CSV12_Accumulated};
	  $output_ref->{$field_pos} = 
            $values{NumActRABMean_CSV12} * 1.0 * $perlen;

          # NumActRABs_CSD_Accumulated
          $field_pos= $ppConfig->{COUNTDDGROUPFIELD}{UtranCell2}{NumActRABs_CSD_Accumulated};
	  $output_ref->{$field_pos} =
            $values{NumActRABMean_CSD} * 1.0 * $perlen;

          my $RAB_RelCS_Voice_CauseRLF_Adjusted;
          my $RAB_RelCS_Data_CauseRLF_Adjusted;
          my $RAB_RelPS_CauseRLF_Adjusted;

          my $divisor = $values{RAB_RelCS_Voice_CauseRLF} +
                        $values{RAB_RelCS_Data_CauseRLF} +
                        $values{RAB_RelPS_DCH_CauseRLF} +
                        $values{RAB_RelPS_HSDSCH_CauseRLF} +
                        $values{RAB_RelPS_HSDSCH_CauseRLF_ReconfFail};
          if ($divisor != 0) {
            $RAB_RelCS_Voice_CauseRLF_Adjusted = $values{RAB_RelCS_Voice_CauseRLF} * 
                                                 $values{RAB_Rel_Drop_sum} * 1.0 /
                                                 $divisor; 
            $RAB_RelCS_Data_CauseRLF_Adjusted = $values{RAB_RelCS_Data_CauseRLF} *
                                                $values{RAB_Rel_Drop_sum} * 1.0 /
                                                $divisor;
            $RAB_RelPS_CauseRLF_Adjusted = ( $values{RAB_RelPS_DCH_CauseRLF} +
                                             $values{RAB_RelPS_HSDSCH_CauseRLF} +
                                             $values{RAB_RelPS_HSDSCH_CauseRLF_ReconfFail} ) *
                                           $values{RAB_Rel_Drop_sum} * 1.0 /
                                           $divisor;
          } else {
            $RAB_RelCS_Voice_CauseRLF_Adjusted='';
            $RAB_RelCS_Data_CauseRLF_Adjusted='';
            $RAB_RelPS_CauseRLF_Adjusted='';
          }

          $field_pos= $ppConfig->{COUNTDDGROUPFIELD}{UtranCell2}{RAB_RelCS_Voice_CauseRLF_Adjusted};
	  $output_ref->{$field_pos} = $RAB_RelCS_Voice_CauseRLF_Adjusted;
          $field_pos= $ppConfig->{COUNTDDGROUPFIELD}{UtranCell2}{RAB_RelCS_Data_CauseRLF_Adjusted};
	  $output_ref->{$field_pos} = $RAB_RelCS_Data_CauseRLF_Adjusted;
          $field_pos= $ppConfig->{COUNTDDGROUPFIELD}{UtranCell2}{RAB_RelPS_CauseRLF_Adjusted};
	  $output_ref->{$field_pos} = $RAB_RelPS_CauseRLF_Adjusted;

        }
      } # if exists ... UtranCell2
    }
  } # End of SEAde57340


} # sub build_data_hash

## --------------- End of build_data_hash sub routine--------

# initModules - provides any initialisation required for preLoadProcess modules
sub initModules {
  my ($rawDataFile, $config, $ppConfig) = @_ ;
  $ppConfig->{CONFIG} = $config ;
  $ppConfig->{ELEMENTLIST} = {} ;
  $ppConfig->{NODEID} = "unspecified" ;
  loadElementList($config,$ppConfig->{ELEMENTLIST});
  $ppConfig->{NODETYPE} = "RNC"; # Hack!
  loadPPConfig($config, $ppConfig) ;
  $config->{PPCONFIG} = $ppConfig ;
} # sub initModules
## -------- End of initModules SubRoutine---------------------


# initialiseConfiguration - setup the configuration parameters for this module
sub initialiseConfiguration {

  my %user_defaults=
  (
    # Regex for pulling year, month, day, hours, minutes and seconds out of
    # a timestamp.
    INPUT_TSTAMP_FORMAT =>
    "(\\d{4})(\\d{2})(\\d{2})(\\d{2})(\\d{2})(\\d{2})",

    # Format to use with (s)printf to output timestamps in the format we want.
    OUTPUT_TSTAMP_FORMAT =>
    "%04d%02d%02d%02d%02d%02d",

    LOGFILENAME => "preProcessLog" . todays_date(),
    );

  register_config_defaults (%user_defaults);

  register_read_only_variables
    ("LOGFILENAME");

  register_mandatory_variables
    ("LOGFILEDIR",
     "INSEP",
     "OUTSEP",
     "NODEBXMLPATTERN",
     "RNCFILENAMEPATTERN",
     "ELEMENTLIST",
     "LOADCONFIGFILE",
    );
}
## -------- End of initialiseConfiguration SubRoutine---------------------



sub loadElementList {
  my ($config, $elementList) = @_ ;
  my ($fileName)= $config->{ELEMENTLIST};
  my ($line) ;
  if (!(open(THEFILE, "$fileName"))) {
    fatalError ($config, "Could not open element list ($fileName) for reading: $!") ;
  }
  
  while (<THEFILE>) {
    chomp;
    next if /^\#/ or /^\s*$/; # Skip comments and blank lines.
    my ($element_name, $sys_id, $element_id, $release, $buff_size, $entity_type, $tech_type) = split ('\t', $_);

    if (not defined $tech_type) {
      messageHandler ($config, 1, "Element list ($fileName) corrupted for element : $element_name   Skipping ...");
      next;
    }

    if ($entity_type eq 'RNC') {
      if ($element_id ne '') {
        $elementList->{$element_name}{ID} = $element_id ;
      } else {
        messageHandler($config, 1, "No element ID defined in element list ($fileName) for element: $element_name  Skipping ...");
      }
    }
  }
  close(THEFILE);
} # sub loadElementList

##------------End of loadElementList module------------------

sub loadPPConfig {
  my ($config, $ppConfig) = @_ ;
  my $fileName= $config->{LOADCONFIGFILE};

  $ppConfig->{GROUPFIELDNAME} = {} ;
  $ppConfig->{REMMAPPING} = {} ; # Required only for remediation
  if (!(open(THEFILE, "$fileName"))) {
    fatalError ($ppConfig, "Could not open ppConfig file ($fileName) for reading: $!") ;
  }

  my $line ;
  while ($line = <THEFILE>) {
    chomp $line ;
    next if ($line =~ /^\#/) ;
    next if ($line eq '') ;
    my ($counterNameData, $counterNameDD, $moid_entity, $measurementGroup,
	$groupField, $modList) = split(/\t/, $line);
    $modList =~ s/\"//g ; # Excel inserts quotes, so remove them
    if (length($counterNameData) > 0) {
      if (not defined $modList) {
        messageHandler($ppConfig, 1, "Invalid line found in group mapping file - skipping",
		       "Line : ", $line );
        next;
      }

      if (exists $ppConfig->{MODLIST}{$measurementGroup} and
          $modList ne $ppConfig->{MODLIST}{$measurementGroup}) {
        messageHandler($ppConfig, 1, "Module list \"$modList\" for ",
                       "counter $counterNameDD is different ",
                       "from previous values for measurement group ",
                       "$measurementGroup - ignoring.");
      } else {
        $ppConfig->{MODLIST}{$measurementGroup}= $modList;
      }

      if (!exists($ppConfig->{GROUPFIELDNAME}{$measurementGroup}{maxField}) ||
         ($groupField > $ppConfig->{GROUPFIELDNAME}{$measurementGroup}{maxField})) {
        $ppConfig->{GROUPFIELDNAME}{$measurementGroup}{maxField} = $groupField ;
      }

      if (exists ($ppConfig->{COUNTGROUPFIELD}{$measurementGroup}{$counterNameData})) {
        messageHandler ($ppConfig, 1, "Group field value $groupField for ",
                        "counter $counterNameDD duplicates a previous ",
                        "value in measurement group $measurementGroup.");
        # We still store it, because this is a way to handle counters
        # being renamed.
      }
      $ppConfig->{COUNTGROUPFIELD}{$measurementGroup}{$counterNameData} = $groupField;

      $ppConfig->{COUNTDDGROUPFIELD}{$measurementGroup}{$counterNameDD} = $groupField;

      $ppConfig->{GROUPFIELDNAME}{$measurementGroup}{field}{$groupField}= $counterNameDD;

      $ppConfig->{COUNTER_TO_GROUP}{$moid_entity}{$counterNameData} = $measurementGroup;
      # This allows for the grouping of counters in the data to be different
      # from the grouping of counters in Prospect.
      $ppConfig->{SHORT_COUNTER_TO_GROUP}{$counterNameDD}= $measurementGroup;
    } # if length $counterNameData > 0
  } # while $line = <THEFILE>
  close(THEFILE);

} # sub loadPPConfig

## ------------End of loadPPConfig module-------------------

sub showGeneral {
  my ($fileName, $measTime, $perlen, $ppConfig, $config, $outsep) = @_ ;
  $measTime= round_tstamp_to_period($config, $measTime, $perlen);
  print join ($outsep,
	      "MEASGROUP_RNC_General",
	      "5",
	      "FILENAME",
	      "TIMESTAMP",
	      "PERLENSEC",
              "LABEL",
              "INSTANCE_NAME",
	      "\n"
	     );
  print join ($outsep,
	      "RNC",
	      "",
	      $fileName,
	      $measTime,
	      $perlen,
              $ppConfig->{RNC_LABEL},
              $ppConfig->{RNC_NAME},
	      "\n"
	     );
} # sub showGeneral

## ------------End of showGeneral SubRoutine---------------


## Subroutine to parse the file name extract the timestamp and RNC_ID
sub parse_filename {
  my ($file_name, $config, $ppConfig) = @_;

  my ($RNC_name, $RNC_id, $meas_time);

  ($RNC_name, $meas_time) = $file_name =~ /$config->{RNCFILENAMEPATTERN}/o;

  if (not defined $meas_time) {
    messageHandler ($config, 1, "Could not extract RNC name and meas time from filename.  Filename : ",$file_name);
    return (undef, undef);
  }

  # Add seconds to the measurement time
  $meas_time = $meas_time . "00";

  if (exists $ppConfig->{ELEMENTLIST}{$RNC_name}) {
    $RNC_id = $ppConfig->{ELEMENTLIST}{$RNC_name}{ID};
  } else {
    messageHandler ($config, 1, "Could not find RNC name in element list.  RNC name: ",$RNC_name);
    return (undef, undef);
  }

  return ($RNC_name, $RNC_id, $meas_time);
}
##--------End of parse_filename-----------------


##Return true to show that the module loaded successfully.
1;
