#!/usr/bin/env perl -w
# @(#) preProcess.pl 1.6.0 05/07/25 13:24:36
# @(#) 


# Force warnings to be fatal. Uncomment after development to fail script
# when a warning such as Argument "" isn't numeric in divide is found, in
# situations where the script should not continue processing.
#$SIG{__WARN__} = sub { die $_[0] } ;

use strict ;
use File::Basename;
use Env;
use Time::Local;
use lib $ENV{LuUTRAN_HOME} . "/vendor/Lucent/UTRAN/1.3/LuUTRAN_loader/Common/scripts"; # for general_functions
use lib $ENV{LuUTRAN_HOME} . "/vendor/Lucent/UTRAN/1.3/LuUTRAN_loader/RNCpm/scripts"; # for general_functions
use PreProcess::PLM_MODULEDEF ;
use modules::config_functions;
use modules::time_functions;

my $SUCCESS=1;
my $DATA=0;
my $reached_the_end= undef;
# We set this just before a successful exit,
# so that the END block knows whether we're exiting successfully or not.

# These are by no means all the signals we could receive,
# but these are the ones I think are most likely.
$SIG{HUP}= $SIG{INT}= $SIG{QUIT}= $SIG{TERM}= \&signal_handler;

my $start_time= time;

my $scriptname= basename $0;

if (@ARGV != 2) { usage("Invalid number of arguments") }

my ($rawDataFile, $configFile)= @ARGV;

my $file_name = basename $rawDataFile;

my $ppConfig = {} ;
my $config = {};


# Read the UTRANconfig.csv file.
initialiseConfiguration();
readConfiguration($configFile, $config, $ppConfig) ;
# After this point, we know what LOG_TRACE_LEVEL is, so we can use
# messageHandler to report errors and warnings.

# Initialise any modules, and read in the ppConfig hash
initModules($rawDataFile, $config, $ppConfig) ;

messageHandler ($ppConfig, 2, "Started processing at: " . epoch_to_tstamp($config, $start_time) );

my $output = {};

# Input the general configuration hash
build_data_hash($ppConfig, $output, $config->{INSEP}, $config);

outputData() ; # print output hash

# End logging
my $end_time= time;
messageHandler ($ppConfig, 2, "Finished processing of file: $rawDataFile, at: " . epoch_to_tstamp($config, $end_time) );
messageHandler ($ppConfig, 2,
		"Elapsed time was ", 
		$end_time - $start_time,
		" second", ($end_time - $start_time == 1 ? "" : "s")
	       );

if ($SUCCESS and $DATA) {
   open(SUCCESS , ">/tmp/Lucent_UTRAN_RNC_preProcess." . getppid);
   close(SUCCESS);
}

$reached_the_end= "yes";
exit(0) ;

# Subroutines #

sub outputData {
  return if (!(keys %{$output})) ; # no data to output
  messageHandler ($ppConfig, 3, "Entering outputData.");

  $DATA=1;
  
  my $outsep = $config->{OUTSEP};

  # Now output the data in output hash
  foreach my $measTime (sort keys %{$output}) {
    
    foreach my $perlen (sort { $a <=> $b } keys %{$output->{$measTime}}) {
      # Display general group heading and data
      messageHandler
	($ppConfig, 3, 
	 "Calling showGeneral with $file_name, $measTime, $perlen, $ppConfig, $outsep"
	);
      showGeneral($file_name, $measTime, $perlen, $ppConfig, $config, $outsep) ;
      foreach my $measGroup (keys %{$output->{$measTime}{$perlen}}) {
        my $maxFields = $ppConfig->{GROUPFIELDNAME}{$measGroup}{maxField} ;
	my $ref= \%{$output->{$measTime}{$perlen}{$measGroup}};
        # Now display measurement group heading
        print 'MEASGROUP_', $measGroup, $outsep, $maxFields, $outsep ;

        # Field names. The loader skips over these (!) but they can be
        # useful for troubleshooting.
        foreach my $idx (1 .. $maxFields) {
          if (defined($ppConfig->{GROUPFIELDNAME}{$measGroup}{field}{$idx})) {
            print $ppConfig->{GROUPFIELDNAME}{$measGroup}{field}{$idx}, $outsep;
	  } else {
	    print $outsep;
	  }
        }
        print "\n" ;

	foreach my $jobIdx (keys %{$ref}) {
	  print $jobIdx, $outsep, $outsep;
	  my $jobIdx_ref= \%{$output->{$measTime}{$perlen}{$measGroup}{$jobIdx}};
	  foreach my $idx (1 .. $maxFields) {
	    if (exists $jobIdx_ref->{$idx} && defined $jobIdx_ref->{$idx}) {
	      print $jobIdx_ref->{$idx}, $outsep;
	    } else {
	      print $outsep;
	    }
	  } # foreach $idx
	  print "\n";
	} # foreach $jobIdx
      } # foreach $measGroup

    } # foreach $perlen
  } # foreach $measTime
  messageHandler ($ppConfig, 3, "Leaving outputData.");
} # sub outputData


sub usage {
  my $message = $_[0] ;

print STDERR <<EOD;


  \nFAIL: $message
  $scriptname: a program to pre-process data.
  USAGE:
  	$scriptname <datafile> <pp_config_file>
  WHERE:
  	<datafile> is the datafile to load
  	<pp_config_file> is a file determining how file should be reformatted

EOD
  exit(1);
} # sub usage



sub signal_handler {
  # Part of fix for LONkw05038 and SEAde48828.
  # We shouldn't receive any signals in normal operation, so if we do,
  # it almost certainly means something has gone wrong.
  my $signal= shift;
  print STDERR "$scriptname: failed: (died from signal $signal)\n";
  exit 1;
}


END {
  # Part of fix for LONkw05038 and SEAde48828.
  # This code is executed if the program wants to exit normally or
  # if it executes a die statement that isn't trapped by an eval.
  # We don't come here if we're killed by a signal, unless the signal
  # is one which we set &signal_handler to deal with.
  unless ($reached_the_end) {
    #print OUT "FAIL (found fatal errors in input)\n";
    print STDERR "$scriptname: failed: $!\n";
    if ($@) { 
      print STDERR "The following may be helpful in diagnosing the problem:\n";
      print STDERR $@;
      print STDERR "(end of possibly helpful information from $scriptname)\n";
    }
    1;
  } else {
    0;
  }
} # end of END block
